from uuid import UUID

import fastapi

from .schemas import (
    TaskPostSchema, TaskPatchSchema, CommentSchema
)
from .ctrl import TaskController
from ..dependencies import get_current_user


router = fastapi.APIRouter(prefix='/tasks', tags=['Task'])
ctrl = TaskController()


@router.get('')
async def get_tasks(
    title: str | None = None,
    status: str | None = None,
    category: str | None = None,
    priority: str | None = None,
    sort_by: str | None = None,
    page: int = 1,
    size: int = 10,
):
    return await ctrl.get_list(
        title=title,
        status=status,
        category=category,
        priority=priority,
        sort_by=sort_by,
        page=page,
        size=size,
    )

@router.get('/{id}')
async def get_task(id: UUID):
    return await ctrl.get(id)


@router.post('')
async def create_task(
    body: TaskPostSchema,
    current_user: str = fastapi.Depends(get_current_user)
):
    return await ctrl.create(**body.model_dump(exclude_none=True))


@router.patch('/{id}')
async def update_task(
    id: UUID,
    body: TaskPatchSchema,
    current_user: str = fastapi.Depends(get_current_user)
):
    return await ctrl.update(id, **body.model_dump(exclude_none=True))


@router.delete('/{id}')
async def delete_task(
    id: UUID,
    current_user: str = fastapi.Depends(get_current_user)
):
    return await ctrl.delete(id)


@router.get('/{id}/comments')
async def get_comments(id: UUID):
    return await ctrl.get_comments(id)


@router.post('/{id}/comments')
async def add_comment(
    id: UUID,
    body: CommentSchema,
    current_user: str = fastapi.Depends(get_current_user)
):
    return await ctrl.add_comment(id, **body.model_dump(exclude_none=True))


@router.delete('/comments/{id}')
async def delete_comment(
    id: UUID,
    current_user: str = fastapi.Depends(get_current_user)
):
    return await ctrl.delete_comment(id)


@router.get('/tasks/search')
async def search_tasks(keywords: str):
    return await ctrl.search(keywords)
